@@ -28,9 +28,12 @@
 #include "gdict-utils.h"
 #include "gdict-marshal.h"
 #include "gdict-context-private.h"
+#include "gdict-private.h"
+
 
 static void gdict_context_class_init (gpointer g_iface);
 
+
 GType
 gdict_context_get_type (void)
 {
@@ -202,12 +205,7 @@ gdict_context_class_init (gpointer g_iface)
 GQuark
 gdict_context_error_quark (void)
 {
-  static GQuark quark = 0;
-  
-  if (G_UNLIKELY (quark == 0))
-    quark = g_quark_from_static_string ("gdict-context-error-quark");
-  
-  return quark;
+  return g_quark_from_static_string ("gdict-context-error-quark");
 }
 
 /**
@@ -257,7 +255,7 @@ gdict_context_get_local_only (GdictContext *context)
  * @error: return location for a #GError, or %NULL
  *
  * Query @context for the list of databases available.  Each time a
- * new database is found, the "database-found" signal is fired.
+ * database is found, the "database-found" signal is fired.
  *
  * Return value: %TRUE if the query was successfully started.
  *
@@ -268,6 +266,15 @@ gdict_context_lookup_databases (GdictContext  *context,
 				GError       **error)
 {
   g_return_val_if_fail (GDICT_IS_CONTEXT (context), FALSE);
+
+  if (!GDICT_CONTEXT_GET_IFACE (context)->get_databases)
+    {
+      g_warning ("Object `%s' does not implement the get_databases "
+                 "virtual function.",
+                 g_type_name (G_OBJECT_TYPE (context)));
+      
+      return FALSE;
+    }
   
   return GDICT_CONTEXT_GET_IFACE (context)->get_databases (context, error);
 }
@@ -289,7 +296,16 @@ gdict_context_lookup_strategies (GdictContext  *context,
 				 GError       **error)
 {
   g_return_val_if_fail (GDICT_IS_CONTEXT (context), FALSE);
-  
+
+  if (!GDICT_CONTEXT_GET_IFACE (context)->get_strategies)
+    {
+      g_warning ("Object `%s' does not implement the get_strategies "
+                 "virtual function.",
+                 g_type_name (G_OBJECT_TYPE (context)));
+      
+      return FALSE;
+    }
+
   return GDICT_CONTEXT_GET_IFACE (context)->get_strategies (context, error);
 }
 
@@ -320,6 +336,15 @@ gdict_context_match_word (GdictContext  *context,
 {
   g_return_val_if_fail (GDICT_IS_CONTEXT (context), FALSE);
   g_return_val_if_fail (word != NULL, FALSE);
+
+  if (!GDICT_CONTEXT_GET_IFACE (context)->match_word)
+    {
+      g_warning ("Object `%s' does not implement the match_word "
+                 "virtual function.",
+                 g_type_name (G_OBJECT_TYPE (context)));
+      
+      return FALSE;
+    }
   
   return GDICT_CONTEXT_GET_IFACE (context)->match_word (context,
   							database,
@@ -352,6 +377,15 @@ gdict_context_define_word (GdictContext  *context,
   g_return_val_if_fail (GDICT_IS_CONTEXT (context), FALSE);
   g_return_val_if_fail (word != NULL, FALSE);
   
+  if (!GDICT_CONTEXT_GET_IFACE (context)->define_word)
+    {
+      g_warning ("Object `%s' does not implement the define_word "
+                 "virtual function.",
+                 g_type_name (G_OBJECT_TYPE (context)));
+      
+      return FALSE;
+    }
+  
   return GDICT_CONTEXT_GET_IFACE (context)->define_word (context,
   							 database,
   							 word,
@@ -373,7 +407,7 @@ _gdict_database_new (const gchar *name)
   
   g_return_val_if_fail (name != NULL, NULL);
   
-  retval = g_new0 (GdictDatabase, 1);
+  retval = g_slice_new (GdictDatabase);
   retval->name = g_strdup (name);
   retval->full_name = NULL;
   retval->ref_count = 1;
@@ -425,7 +459,7 @@ gdict_database_unref (GdictDatabase *db)
       g_free (db->name);
       g_free (db->full_name);
       
-      g_free (db);
+      g_slice_free (GdictDatabase, db);
     }
 }
 
@@ -483,7 +517,7 @@ _gdict_strategy_new (const gchar *name)
   
   g_return_val_if_fail (name != NULL, NULL);
   
-  strat = g_new0 (GdictStrategy, 1);
+  strat = g_slice_new (GdictStrategy);
   strat->name = g_strdup (name);
   strat->description = NULL;
   strat->ref_count = 1;
@@ -536,10 +570,20 @@ gdict_strategy_unref (GdictStrategy *strat)
       g_free (strat->name);
       g_free (strat->description);
       
-      g_free (strat);
+      g_slice_free (GdictStrategy, strat);
     }
 }
 
+/**
+ * gdict_strategy_get_name:
+ * @strat: a #GdictStrategy
+ *
+ * FIXME
+ *
+ * Return value: FIXME
+ *
+ * Since: 1.0
+ */
 G_CONST_RETURN gchar *
 gdict_strategy_get_name (GdictStrategy *strat)
 {
@@ -548,6 +592,16 @@ gdict_strategy_get_name (GdictStrategy *strat)
   return strat->name;
 }
 
+/**
+ * gdict_strategy_get_description:
+ * @strat: a #GdictStrategy
+ *
+ * FIXME
+ *
+ * Return value: FIXME
+ *
+ * Since: 1.0
+ */
 G_CONST_RETURN gchar *
 gdict_strategy_get_description (GdictStrategy *strat)
 {
@@ -571,7 +625,7 @@ _gdict_match_new (const gchar *word)
   
   g_return_val_if_fail (word != NULL, NULL);
   
-  match = g_new0 (GdictMatch, 1);
+  match = g_slice_new (GdictMatch);
   match->word = g_strdup (word);
   match->database = NULL;
   match->ref_count = 1;
@@ -579,6 +633,16 @@ _gdict_match_new (const gchar *word)
   return match;
 }
 
+/**
+ * gdict_match_ref:
+ * @match: a #GdictMatch
+ *
+ * FIXME
+ *
+ * Return value: FIXME
+ *
+ * Since: 1.0
+ */
 GdictMatch *
 gdict_match_ref (GdictMatch *match)
 {
@@ -591,6 +655,14 @@ gdict_match_ref (GdictMatch *match)
   return match;
 }
 
+/**
+ * gdict_match_unref:
+ * @match: a #GdictMatch
+ *
+ * FIXME
+ *
+ * Since: 1.0
+ */
 void
 gdict_match_unref (GdictMatch *match)
 {
@@ -605,10 +677,20 @@ gdict_match_unref (GdictMatch *match)
       g_free (match->word);
       g_free (match->database);
       
-      g_free (match);
+      g_slice_free (GdictMatch, match);
     }
 }
 
+/**
+ * gdict_match_get_word:
+ * @match: a #GdictMatch
+ *
+ * FIXME
+ *
+ * Return value: FIXME
+ *
+ * Since: 1.0
+ */
 G_CONST_RETURN gchar *
 gdict_match_get_word (GdictMatch *match)
 {
@@ -617,6 +699,16 @@ gdict_match_get_word (GdictMatch *match)
   return match->word;
 }
 
+/**
+ * gdict_match_get_database:
+ * @match: a #GdictMatch
+ *
+ * FIXME
+ *
+ * Return value: FIXME
+ *
+ * Since: 1.0
+ */
 G_CONST_RETURN gchar *
 gdict_match_get_database (GdictMatch *match)
 {
@@ -639,7 +731,7 @@ _gdict_definition_new (gint total)
 {
   GdictDefinition *def;
   
-  def = g_new0 (GdictDefinition, 1);
+  def = g_slice_new (GdictDefinition);
   
   def->total = total;
   def->word = NULL;
@@ -696,7 +788,7 @@ gdict_definition_unref (GdictDefinition *def)
       g_free (def->database_name);
       g_free (def->database_full);
       
-      g_free (def);
+      g_slice_free (GdictDefinition, def);
     }
 }
 
