@@ -10,12 +10,15 @@
 #include <glade/glade.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
+enum {
+  TYPE_IMAGE_PNG,
+  TYPE_TEXT_URI_LIST,
+};
 
 static GtkTargetEntry drag_types[] =
 {
-  { "image/png", 0, 0 },
-  { "x-special/gnome-icon-list", 0, 0 },
-  { "text/uri-list", 0, 0 }
+  { "image/png", 0, TYPE_IMAGE_PNG },
+  { "text/uri-list", 0, TYPE_TEXT_URI_LIST }
 };
 
 struct ScreenshotDialog
@@ -141,16 +144,28 @@ drag_data_get (GtkWidget          *widget,
 	       GtkSelectionData   *selection_data,
 	       guint               info,
 	       guint               time,
-	       gpointer            data)
+	       ScreenshotDialog *dialog)
 {
-	char *string;
+  if (info == TYPE_TEXT_URI_LIST)
+    {
+      gchar **uris;
+      
+      uris = g_new (gchar *, 2);
+      uris[0] = g_strconcat ("file://",
+                             screenshot_save_get_filename (),
+                             NULL);
+      uris[1] = NULL;
 
-	string = g_strdup_printf ("file:%s\r\n",
-				  screenshot_save_get_filename ());
-	gtk_selection_data_set (selection_data,
-				selection_data->target,
-				8, string, strlen (string)+1);
-	g_free (string);
+      gtk_selection_data_set_uris (selection_data, uris);
+    }
+  else if (info == TYPE_IMAGE_PNG)
+    {
+      gtk_selection_data_set_pixbuf (selection_data, dialog->screenshot);
+    }
+  else
+    {
+      g_warning ("Unknown type %d", info);
+    }
 }
 
 static void
@@ -256,7 +271,7 @@ screenshot_dialog_new (GdkPixbuf *screenshot,
    */
   ext = g_utf8_strrchr (current_name, -1, '.');
   if (ext)
-    pos = ext - current_name;
+    pos = g_utf8_strlen (current_name, -1) - g_utf8_strlen (ext, -1);
   else
     pos = -1;
 
@@ -278,7 +293,7 @@ screenshot_dialog_enable_dnd (ScreenshotDialog *dialog)
 
   preview_darea = glade_xml_get_widget (dialog->xml, "preview_darea");
   gtk_drag_source_set (preview_darea,
-		       GDK_BUTTON1_MASK|GDK_BUTTON3_MASK,
+		       GDK_BUTTON1_MASK | GDK_BUTTON3_MASK,
 		       drag_types, G_N_ELEMENTS (drag_types),
 		       GDK_ACTION_COPY);
 }
